/*
   (c) 2021 Christian Grieger
   GNU GENERAL PUBLIC LICENSE
*/

#include <TrafficLights.h>

const byte TrafficLights::RED    = 0;
const byte TrafficLights::YELLOW = 1;
const byte TrafficLights::GREEN  = 2;

TrafficLights::TrafficLights (byte pinRed, byte pinYellow, byte pinGreen, byte delayFactor, byte initialDelay)
{
  pins[RED] = pinRed;
  pins[YELLOW] = pinYellow;
  pins[GREEN] = pinGreen;
  for (byte i = 0; i < sizeof(pins); i++) {
    pinMode(pins[i], OUTPUT);
  }

  // all delays in seconds
  delays[RED] = 10 * delayFactor;
  delays[YELLOW] = 1;
  delays[GREEN] = 8 * delayFactor;

  currentIndex = RED;
  nextAction = (long)millis() + delays[currentIndex] * 1000 + initialDelay*1000;
}

void TrafficLights::update()
{
  if ((long)millis() - nextAction >= 0) {
    digitalWrite(pins[currentIndex], LOW);
    switch(currentIndex) {
      case RED:
        currentIndex = GREEN;
        break;      
      case YELLOW:
        currentIndex = RED;
        break;      
      case GREEN:
        currentIndex = YELLOW;
        break;      
    }
    
    nextAction = (long)millis() + delays[currentIndex] * 1000;
    digitalWrite(pins[currentIndex], HIGH);
  }
}
